@@ -5,12 +5,14 @@ import os
 import re
 import sys
 from dataclasses import dataclass
+from datetime import datetime
 from functools import lru_cache
 from pathlib import Path
-from subprocess import CalledProcessError, check_output
+from subprocess import DEVNULL, PIPE, CalledProcessError, check_output, run
 from typing import Iterable, List, Set
 
 from darker.diff import diff_and_get_opcodes, opcodes_to_edit_linenums
+from darker.utils import GIT_DATEFORMAT, TextDocument
 
 logger = logging.getLogger(__name__)
 
@@ -30,7 +32,20 @@ WORKTREE = ":WORKTREE:"
 PRE_COMMIT_FROM_TO_REFS = ":PRE-COMMIT:"
 
 
-def git_get_content_at_revision(path: Path, revision: str, cwd: Path) -> List[str]:
+def git_get_mtime_at_commit(path: Path, revision: str, cwd: Path) -> str:
+    """Return the committer date of the given file at the given revision
+
+    :param path: The relative path of the file in the Git repository
+    :param revision: The Git revision for which to get the file modification time
+    :param cwd: The root of the Git repository
+
+    """
+    cmd = ["log", "-1", "--format=%ct", revision, "--", path.as_posix()]
+    lines = _git_check_output_lines(cmd, cwd)
+    return datetime.utcfromtimestamp(int(lines[0])).strftime(GIT_DATEFORMAT)
+
+
+def git_get_content_at_revision(path: Path, revision: str, cwd: Path) -> TextDocument:
     """Get unmodified text lines of a file at a Git revision
 
     :param path: The relative path of the file in the Git repository
@@ -39,19 +54,28 @@ def git_get_content_at_revision(path: Path, revision: str, cwd: Path) -> List[st
     :param cwd: The root of the Git repository
 
     """
+    assert (
+        not path.is_absolute()
+    ), f"the 'path' parameter must receive a relative path, got {path!r} instead"
+
     if revision == WORKTREE:
-        return (cwd / path).read_text("utf-8").splitlines()
-    cmd = ["git", "show", f"{revision}:./{path}"]
+        abspath = cwd / path
+        return TextDocument.from_file(abspath)
+    cmd = ["show", f"{revision}:./{path.as_posix()}"]
     logger.debug("[%s]$ %s", cwd, " ".join(cmd))
     try:
-        return check_output(cmd, cwd=str(cwd), encoding="utf-8").splitlines()
+        return TextDocument.from_lines(
+            _git_check_output_lines(cmd, cwd, exit_on_error=False),
+            mtime=git_get_mtime_at_commit(path, revision, cwd),
+        )
     except CalledProcessError as exc_info:
-        if exc_info.returncode == 128:
-            # The file didn't exist at the given revision. Act as if it was an empty
-            # file, so all current lines appear as edited.
-            return []
-        else:
+        if exc_info.returncode != 128:
+            for error_line in exc_info.stderr.splitlines():
+                logger.error(error_line)
             raise
+        # The file didn't exist at the given revision. Act as if it was an empty
+        # file, so all current lines appear as edited.
+        return TextDocument()
 
 
 @dataclass(frozen=True)
@@ -101,12 +125,8 @@ class RevisionRange:
                     use_common_ancestor=True,
                 )
             except KeyError:
-                logger.error(
-                    "The environment variables PRE_COMMIT_FROM_REF and"
-                    " PRE_COMMIT_TO_REF must be defined when using -r / --revision"
-                    " :PRE-COMMIT:"
-                )
-                sys.exit(123)
+                # Fallback to running against HEAD
+                revision_range = "HEAD"
         match = COMMIT_RANGE_RE.match(revision_range)
         if match:
             rev1, range_dots, rev2 = match.groups()
@@ -123,17 +143,63 @@ def should_reformat_file(path: Path) -> bool:
     return path.exists() and path.suffix == ".py"
 
 
-def _git_check_output_lines(cmd: List[str], cwd: Path) -> List[str]:
+def _git_check_output_lines(
+    cmd: List[str], cwd: Path, exit_on_error: bool = True
+) -> List[str]:
     """Log command line, run Git, split stdout to lines, exit with 123 on error"""
     logger.debug("[%s]$ %s", cwd, " ".join(cmd))
     try:
-        return check_output(cmd, cwd=str(cwd)).decode("utf-8").splitlines()
+        return check_output(
+            ["git"] + cmd,
+            cwd=str(cwd),
+            encoding="utf-8",
+            stderr=PIPE,
+            env={"LC_ALL": "C"},
+        ).splitlines()
     except CalledProcessError as exc_info:
-        if exc_info.returncode == 128:
-            # Bad revision or another Git failure
-            sys.exit(123)
-        else:
+        if not exit_on_error:
             raise
+        if exc_info.returncode != 128:
+            sys.stderr.write(exc_info.stderr)
+            raise
+
+        # Bad revision or another Git failure. Follow Black's example and return the
+        # error status 123.
+        for error_line in exc_info.stderr.splitlines():
+            logger.error(error_line)
+        sys.exit(123)
+
+
+def _git_exists_in_revision(path: Path, rev2: str) -> bool:
+    """Return ``True`` if the given path exists in the given Git revision
+
+    :param path: The path of the file or directory to check
+    :param rev2: The Git revision to look at
+    :return: ``True`` if the file or directory exists at the revision, or ``False`` if
+             it doesn't.
+
+    """
+    # Surprise: On Windows, `git cat-file` doesn't work with backslash directory
+    # separators in paths. We need to use Posix paths and forward slashes instead.
+    cmd = ["git", "cat-file", "-e", f"{rev2}:{path.as_posix()}"]
+    result = run(cmd, check=False, stderr=DEVNULL, env={"LC_ALL": "C"})
+    return result.returncode == 0
+
+
+def get_missing_at_revision(paths: Iterable[Path], rev2: str) -> Set[Path]:
+    """Return paths missing in the given revision
+
+    In case of ``WORKTREE``, just check if the files exist on the filesystem instead of
+    asking Git.
+
+    :param paths: Paths to check
+    :param rev2: The Git revision to look at, or ``WORKTREE`` for the working tree
+    :return: The set of file or directory paths which are missing in the revision
+
+    """
+    if rev2 == WORKTREE:
+        return {path for path in paths if not path.exists()}
+    return {path for path in paths if not _git_exists_in_revision(path, rev2)}
 
 
 def git_get_modified_files(
@@ -152,15 +218,14 @@ def git_get_modified_files(
 
     """
     relative_paths = {p.resolve().relative_to(cwd) for p in paths}
-    str_paths = [str(path) for path in relative_paths]
+    str_paths = [path.as_posix() for path in relative_paths]
     if revrange.use_common_ancestor:
         rev2 = "HEAD" if revrange.rev2 == WORKTREE else revrange.rev2
-        merge_base_cmd = ["git", "merge-base", revrange.rev1, rev2]
+        merge_base_cmd = ["merge-base", revrange.rev1, rev2]
         rev1 = _git_check_output_lines(merge_base_cmd, cwd)[0]
     else:
         rev1 = revrange.rev1
     diff_cmd = [
-        "git",
         "diff",
         "--name-only",
         "--relative",
@@ -174,7 +239,6 @@ def git_get_modified_files(
     lines = _git_check_output_lines(diff_cmd, cwd)
     if revrange.rev2 == WORKTREE:
         ls_files_cmd = [
-            "git",
             "ls-files",
             "--others",
             "--exclude-standard",
@@ -196,24 +260,22 @@ class EditedLinenumsDiffer:
     @lru_cache(maxsize=1)
     def compare_revisions(self, path_in_repo: Path, context_lines: int) -> List[int]:
         """Return numbers of lines changed between a given revision and the worktree"""
-        lines = git_get_content_at_revision(
-            path_in_repo, self.revrange.rev2, self.git_root
-        )
-        return self.revision_vs_lines(path_in_repo, lines, context_lines)
+        content = TextDocument.from_file(self.git_root / path_in_repo)
+        return self.revision_vs_lines(path_in_repo, content, context_lines)
 
     def revision_vs_lines(
-        self, path_in_repo: Path, lines: List[str], context_lines: int
+        self, path_in_repo: Path, content: TextDocument, context_lines: int
     ) -> List[int]:
         """For file `path_in_repo`, return changed line numbers from given revision
 
         :param path_in_repo: Path of the file to compare, relative to repository root
-        :param lines: The contents to compare to, e.g. from current working tree
+        :param content: The contents to compare to, e.g. from current working tree
         :param context_lines: The number of lines to include before and after a change
         :return: Line numbers of lines changed between the revision and given content
 
         """
-        revision_lines = git_get_content_at_revision(
+        old = git_get_content_at_revision(
             path_in_repo, self.revrange.rev1, self.git_root
         )
-        edited_opcodes = diff_and_get_opcodes(revision_lines, lines)
+        edited_opcodes = diff_and_get_opcodes(old, content)
         return list(opcodes_to_edit_linenums(edited_opcodes, context_lines))
