@@ -1,50 +1,45 @@
 """Configuration and fixtures for the Pytest based test suite"""
 
-import sys
-import types
+import os
 from pathlib import Path
 from subprocess import check_call
 from typing import Dict, Optional
-from unittest.mock import patch
 
 import pytest
 from black import find_project_root
-from py.path import local as LocalPath
 
 from darker.git import _git_check_output_lines
 
 
-@pytest.fixture
-def without_isort():
-    with patch.dict(sys.modules, {"isort": None}):
-        yield
-
-
-@pytest.fixture
-def with_isort():
-    with patch.dict(sys.modules, {"isort": types.ModuleType("isort")}):
-        yield
-
-
 class GitRepoFixture:
-    def __init__(self, root: LocalPath):
+    def __init__(self, root: Path, env: Dict[str, str]):
         self.root = root
+        self.env = env
 
     @classmethod
-    def create_repository(cls, root: LocalPath) -> "GitRepoFixture":
+    def create_repository(cls, root: Path) -> "GitRepoFixture":
         """Fixture method for creating a Git repository in the given directory"""
-        check_call(["git", "init"], cwd=root)
-        check_call(["git", "config", "user.email", "ci@example.com"], cwd=root)
-        check_call(["git", "config", "user.name", "CI system"], cwd=root)
-        return cls(root)
+        env = os.environ.copy()
+        # for testing, ignore ~/.gitconfig settings like templateDir and defaultBranch
+        env["HOME"] = str(root)
+        instance = cls(root, env)
+        # pylint: disable=protected-access
+        instance._run("init")
+        instance._run("config", "user.email", "ci@example.com")
+        instance._run("config", "user.name", "CI system")
+        return instance
 
     def _run(self, *args: str) -> None:
         """Helper method to run a Git command line in the repository root"""
-        check_call(["git"] + list(args), cwd=self.root)
+        check_call(["git"] + list(args), cwd=self.root, env=self.env)
+
+    def _run_and_get_first_line(self, *args: str) -> str:
+        """Helper method to run Git in repo root and return first line of output"""
+        return _git_check_output_lines(list(args), Path(self.root))[0]
 
     def add(
         self, paths_and_contents: Dict[str, Optional[str]], commit: str = None
-    ) -> Dict[str, LocalPath]:
+    ) -> Dict[str, Path]:
         """Add/remove/modify files and optionally commit the changes
 
         :param paths_and_contents: Paths of the files relative to repository root, and
@@ -63,15 +58,20 @@ class GitRepoFixture:
             if content is None:
                 self._run("rm", "--", relative_path)
             else:
-                path.write(content, ensure=True)
+                path.parent.mkdir(parents=True, exist_ok=True)
+                path.write_bytes(content.encode("utf-8"))
                 self._run("add", "--", relative_path)
         if commit:
             self._run("commit", "-m", commit)
         return absolute_paths
 
-    def get_hash(self) -> str:
-        """Return the commit hash at HEAD in the Git repository"""
-        return _git_check_output_lines(["git", "rev-parse", "HEAD"], Path(self.root))[0]
+    def get_hash(self, revision: str = "HEAD") -> str:
+        """Return the commit hash at the given revision in the Git repository"""
+        return self._run_and_get_first_line("rev-parse", revision)
+
+    def get_branch(self) -> str:
+        """Return the active branch name in the Git repository"""
+        return self._run_and_get_first_line("symbolic-ref", "--short", "HEAD")
 
     def create_branch(self, new_branch: str, start_point: str) -> None:
         """Fixture method to create and check out new branch at given starting point"""
@@ -79,10 +79,10 @@ class GitRepoFixture:
 
 
 @pytest.fixture
-def git_repo(tmpdir, monkeypatch):
+def git_repo(tmp_path, monkeypatch):
     """Create a temporary Git repository and change current working directory into it"""
-    repository = GitRepoFixture.create_repository(tmpdir)
-    monkeypatch.chdir(tmpdir)
+    repository = GitRepoFixture.create_repository(tmp_path)
+    monkeypatch.chdir(tmp_path)
     return repository
 
 
